//+------------------------------------------------------------------+
//|                                                     MAProfit.mq4 |
//|                      Copyright  2010, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010, Thomas Quester 'tflores'."
#property link      "tquester@gmx.de"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1 MediumSpringGreen
#property indicator_color2 Yellow
//---- input parameters
extern int       PeriodShort=6;
extern int       PeriodLong=40;
extern int       Method=1;
extern bool      Optimize=true;
extern bool      DrawTringles=true;
extern int       MinShortMA=2;
extern int       MaxShortMA=20;
extern int       MaxLongMA=100;
extern int       StepLongMA=3;
extern int       StepShortMA=3;
extern int       CountOptimize=200;
extern bool      OptimizeOnNewCandle=false;
extern bool      Alarm=true;

string AlertShort = "alert.wav";
string AlertLong  = "alert.wav";

//---- buffers
double ExtMapBuffer1[];
double ExtMapBuffer2[];
int OldPeriod;
int spread;
int cBars;
string OldSymbol;


#define SIGNAL_NONE  0
#define SIGNAL_SHORT 1
#define SIGNAL_LONG  2

int               MaxObj = 0;
int               yesterday,today;        // index of yesterday and today
 

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   OldPeriod = -1;
   OldSymbol = "";

   SetIndexStyle(0,DRAW_LINE,0,1);
   SetIndexBuffer(0,ExtMapBuffer1);
   SetIndexStyle(1,DRAW_LINE,0,0);
   SetIndexBuffer(1,ExtMapBuffer2);
   makelabel("profit",              0,20,"Profit Total",White);   
   makelabel("profityesterday",     300,20,"Profit Total",White);   
   makelabel("signal",              500,20,"Signal",White);   
   cBars = 0;
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
    DeleteObjects();  
//----
   return(0);
  }
  

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   int    counted_bars=IndicatorCounted();
   spread = MarketInfo(Symbol(),MODE_SPREAD);
//----
//----
   int         i;                      // some running integer 
   int         objid;                  // the number of triangles
   string      name;                   // name of triangle
   double      d;                      // some double 
   double      maS,maL;                // moving average small and long value
   double      profit = 0;             // profit of "trade" as difference between open and close
   double      open;                   // the open price
   int         day;                    // the day
   datetime    opentime;               // the time of open trade
   int         openid;                 // the index of open trade
   int         signal,s;               // active and new singal
   double      totalProfit;            // total profit
   double      totalProfitYesterday;    // total profit yesterday
   bool        newBar;
   
   signal = SIGNAL_NONE;
   if (Bars != cBars)
       newBar = true;
   else  
       newBar = false;
   cBars = Bars;

   if (Period() != OldPeriod || Symbol() != OldSymbol && Optimize || (OptimizeOnNewCandle && newBar))
      Optimize();
   objid = 0;
   DeleteObjects();                    // delete old objects
   
   // when starts today?
   CalcDays();
   totalProfit           = 0;
   totalProfitYesterday  = 0;
   Print("today="+today);
   Print("yesterday="+yesterday);
   ObjectCreate("today",OBJ_VLINE,0,Time[today],0);
   ObjectCreate("yesterday",OBJ_VLINE,0,Time[yesterday],0);
   for (i=Bars;i>=0;i--)
   {
      maS = iMA(NULL,NULL,PeriodShort,0,Method,PRICE_MEDIAN,i);
      maL = iMA(NULL,NULL,PeriodLong,0,Method,PRICE_MEDIAN,i);
      d = maS - maL;
      d /= Point;
      ExtMapBuffer1[i] = d;
      ExtMapBuffer2[i] = 0;
      
      s = SIGNAL_NONE;
          
      if (maS < maL) s = SIGNAL_SHORT; 
      if (maS > maL) s = SIGNAL_LONG;
      if (s != signal)
      {
          
          profit = 0;
          if (signal == SIGNAL_SHORT) 
          {
               if (newBar && Alarm && i<=1) PlaySound(AlertShort);
               profit = open-Open[i];
               profit /= Point;
               profit -= spread;
          }
          if (signal == SIGNAL_LONG)  
          {
               if (newBar && Alarm && i<=1) PlaySound(AlertLong);
               profit = Open[i]-open;
               profit /= Point;
               profit -= spread;
          }
          
          if (signal != SIGNAL_NONE)
          {
              objid++;
              DrawTriangle(objid,opentime,Open[openid],Time[i],Open[i],profit);              
          }
          
          ExtMapBuffer2[i] = profit;
          if (i <= today)                    totalProfit+=profit;
          if (i <= yesterday && i > today)   totalProfitYesterday+=profit;
          opentime = Time[i];
          openid = i;
          signal = s;
          open = Open[i];
      }
          
      profit = 0;
   }
   
   // terminate open "trade"
      
   if (signal != SIGNAL_NONE)
   {
        i = 0;
        if (signal == SIGNAL_SHORT) 
        {
            profit = open-Open[i];
            profit /= Point;
            profit -= spread;
            SetText("signal","Short");
          }
            
        if (signal == SIGNAL_LONG)  
        {
               profit = Open[i]-open;
               profit /= Point;
               profit -= spread;
               SetText("signal","Long");
        }
        Print("Final Profit " + profit);
        ExtMapBuffer2[i] = profit;
        totalProfit+=profit;
        objid++;
        DrawTriangle(objid,opentime,Open[openid],Time[i],Open[i],profit);
        
    }
    
    i = totalProfit;
    SetText("profit","Profit totay with MA "+PeriodShort+"/"+PeriodLong+" is "+i+" Pips");
    i = totalProfitYesterday;
    SetText("profityesterday","Profit yesterday is "+i+" Pips");
   
   MaxObj=objid+1;
//----
   return(0);
  }
//+------------------------------------------------------------------+


double CalcProfit(int bars, int periodLong, int periodShort)
{
   double      maS,maL;                // moving average small and long value
   double      profit = 0;             // profit of "trade" as difference between open and close
   double      totalProfit;
   int         i,opentime,openid;
   double      d,open;
   int         s,signal;
   signal = SIGNAL_NONE;
   totalProfit           = 0;
   open = 0;
   for (i=bars;i>=0;i--)
   {
      maS = iMA(NULL,NULL,periodShort,0,Method,PRICE_MEDIAN,i);
      maL = iMA(NULL,NULL,periodLong,0,Method,PRICE_MEDIAN,i);
      s = SIGNAL_NONE;
      if (maS < maL) s = SIGNAL_SHORT; 
      if (maS > maL) s = SIGNAL_LONG;
      if (s != signal)
      {
          profit = 0;
          if (signal == SIGNAL_SHORT) 
          {
            profit = open-Open[i];
            profit /= Point;
            profit -= spread;
            totalProfit += profit;
          }
            
          if (signal == SIGNAL_LONG)  
          {
            profit = Open[i]-open;
            profit /= Point;
            profit -= spread;
            totalProfit += profit;
          }
            
          opentime = Time[i];
          openid = i;
          signal = s;
          open = Open[i];
      }
   }
   
      
   if (signal != SIGNAL_NONE)
   {
      profit = 0;
      i = 0;
      if (signal == SIGNAL_SHORT) 
      {
         profit = open-Open[i];
         profit /= Point;
         profit -= spread;
         totalProfit += profit;
      }
         
      if (signal == SIGNAL_LONG)  
      {
         profit = Open[i]-open;
         profit /= Point;
         profit -= spread;
         totalProfit += profit;
      }
         
      
      
        
    }
    return (totalProfit);
}

void Optimize()
{
    double profit, maxProfit;
    int s,l;
    int bestS,bestL;
    bestS = 0;
    bestL = 0;
    maxProfit=0;
    for (s = MinShortMA; s<=MaxShortMA;s+=StepShortMA)
    {
        for (l = s+5;l<=MaxLongMA;l+=StepLongMA)
        {
           profit = CalcProfit(CountOptimize,l,s);
//           if (profit != 0) Print("Profit with "+s+" "+l+" = ",profit);
           if (profit > maxProfit && profit != 0)
           {
              bestS = s;
              bestL = l;
              maxProfit = profit;
           }
        }
    }
    PeriodShort = bestS;
    PeriodLong   = bestL;
}

void CalcDays()
{
   int day;
   int i;
   day = TimeDay(Time[0]);
   for (i=0;i<Bars;i++)
   {
      if (TimeDay(Time[i]) != day)
      {
          today = i-1;
          break;
      }
   }
   // when starts yesterday 
   day = TimeDay(Time[today+1]);
   for (i=today+1;i<Bars;i++)
   {
      if (TimeDay(Time[i]) != day)
      {
          yesterday = i-1;
          break;
      }
   }
}

void DrawTriangle(int objid,datetime opentime,double openprice,datetime timenow,double pricenow,double profit)
{
  string name;
  int i;
  if (DrawTringles)
  {
      name = "profit"+objid;
      ObjectCreate(name,OBJ_TRIANGLE,0,opentime,openprice,timenow,openprice,timenow,pricenow);
      if (profit < 0)
          ObjectSet(name,OBJPROP_COLOR,Red);
      else
          ObjectSet(name,OBJPROP_COLOR,MediumSpringGreen);
      ObjectSet(name,OBJPROP_BACK,false);
      ObjectSet(name,OBJPROP_WIDTH,2);
  }
  name = "win"+objid;
  i = profit;
  ObjectCreate(name, OBJ_TEXT, 0, timenow, pricenow-20*Point);
  ObjectSetText(name, i+" Pips", 8, "Tahoma", White);
  
}

void DeleteObjects()
{
   int i;
   string name;
   for (i=0;i<MaxObj;i++)
   {
      name = "profit"+i;
      ObjectDelete(name);
      name = "win"+i;
      ObjectDelete(name);
   }
   MaxObj = 0;
 }

void makelabel(string lblname,int x,int y,
   string txt,color txtcolor){
   ObjectCreate(lblname, OBJ_LABEL,0, 0, 0);
   ObjectSet(lblname, OBJPROP_CORNER, 0);
   ObjectSetText(lblname,txt,7,"Verdana", txtcolor);
   ObjectSet(lblname, OBJPROP_XDISTANCE, x);
   ObjectSet(lblname, OBJPROP_YDISTANCE, y);
}

void SetText(string name, string txt)
{
   ObjectSetText(name,txt,7,"Verdana", White);
}

